@@ -14,21 +14,23 @@ from prompt_toolkit.buffer import Buffer
 from prompt_toolkit.document import Document
 from prompt_toolkit.enums import DEFAULT_BUFFER
 from prompt_toolkit.filters import Condition, Always
-from prompt_toolkit.history import FileHistory, History
+from prompt_toolkit.history import FileHistory, InMemoryHistory
 from prompt_toolkit.interface import CommandLineInterface, Application, AcceptAction
 from prompt_toolkit.key_binding.manager import KeyBindingManager
+from prompt_toolkit.layout.lexers import PygmentsLexer
 from prompt_toolkit.utils import Callback, is_windows
 from prompt_toolkit.validation import SwitchableValidator
 
 from ptpython.completer import PythonCompleter
 from ptpython.key_bindings import load_python_bindings, load_sidebar_bindings, load_confirm_exit_bindings
-from ptpython.layout import PythonPrompt, create_layout, CompletionVisualisation
+from ptpython.layout import create_layout, CompletionVisualisation
 from ptpython.style import get_all_code_styles, get_all_ui_styles, generate_style
 from ptpython.utils import get_jedi_script_from_document, document_is_multiline_python
 from ptpython.validator import PythonValidator
+from ptpython.prompt_style import IPythonPrompt, ClassicPrompt
 
-from pygments.lexers import PythonLexer
 from functools import partial
+from pygments.lexers import PythonLexer
 
 import six
 import __future__
@@ -121,10 +123,11 @@ class PythonInput(object):
                  vi_mode=False,
 
                  # For internal use.
-                 _completer=None, _validator=None, _python_prompt_control=None,
+                 _completer=None, _validator=None,
                  _lexer=None, _extra_buffers=None, _extra_buffer_processors=None,
                  _on_start=None,
-                 _extra_sidebars=None,
+                 _extra_layout_body=None, _extra_toolbars=None,
+                 _input_buffer_height=None,
                  _accept_action=AcceptAction.RETURN_DOCUMENT,
                  _on_exit=AbortAction.RAISE_EXCEPTION):
 
@@ -133,22 +136,23 @@ class PythonInput(object):
 
         self._completer = _completer or PythonCompleter(self.get_globals, self.get_locals)
         self._validator = _validator or PythonValidator(self.get_compiler_flags)
-        self._history = FileHistory(history_filename) if history_filename else History()
-        self._lexer = _lexer or PythonLexer
+        self.history = FileHistory(history_filename) if history_filename else InMemoryHistory()
+        self._lexer = _lexer or PygmentsLexer(PythonLexer)
         self._extra_buffers = _extra_buffers
         self._accept_action = _accept_action
         self._on_exit = _on_exit
         self._on_start = _on_start
 
-        self._extra_sidebars = _extra_sidebars or []
+        self._input_buffer_height = _input_buffer_height
+        self._extra_layout_body = _extra_layout_body or []
+        self._extra_toolbars = _extra_toolbars or []
         self._extra_buffer_processors = _extra_buffer_processors or []
 
-        self._python_prompt_control = _python_prompt_control or PythonPrompt(self)
-
         # Settings.
         self.show_signature = True
         self.show_docstring = False
-        self.completion_visualisation = CompletionVisualisation.POP_UP
+        self.show_meta_enter_message = True
+        self.completion_visualisation = CompletionVisualisation.MULTI_COLUMN
         self.completion_menu_scroll_offset = 1
 
         self.show_line_numbers = True
@@ -167,10 +171,26 @@ class PythonInput(object):
                                             # history on the records starting
                                             # with the current input.
 
+        self.highlight_matching_parenthesis = True
         self.show_sidebar = False  # Currently show the sidebar.
         self.show_sidebar_help = True # When the sidebar is visible, also show the help text.
         self.show_exit_confirmation = False  # Currently show 'Do you really want to exit?'
         self.terminal_title = None  # The title to be displayed in the terminal. (None or string.)
+        self.exit_message = 'Do you really want to exit?'
+
+        # Tokens to be shown at the prompt.
+        self.prompt_style = 'ipython'  # The currently active style.
+
+        self.all_prompt_styles = {  # Styles selectable from the menu.
+            'ipython': IPythonPrompt(self),
+            'classic': ClassicPrompt(),
+        }
+
+        self.get_input_prompt_tokens = lambda cli: \
+            self.all_prompt_styles[self.prompt_style].in_tokens(cli)
+
+        self.get_output_prompt_tokens = lambda cli: \
+            self.all_prompt_styles[self.prompt_style].out_tokens(cli)
 
         #: Load styles.
         self.code_styles = get_all_code_styles()
@@ -397,6 +417,10 @@ class PythonInput(object):
                            CompletionVisualisation.MULTI_COLUMN: lambda: enable('completion_visualisation', CompletionVisualisation.MULTI_COLUMN),
                            CompletionVisualisation.TOOLBAR: lambda: enable('completion_visualisation', CompletionVisualisation.TOOLBAR),
                        }),
+                Option(title='Prompt',
+                       description='Visualisation of the prompt.',
+                       get_current_value=lambda: self.prompt_style,
+                       get_values=lambda: dict((s, partial(enable, 'prompt_style', s)) for s in self.all_prompt_styles)),
                 simple_option(title='Show signature',
                               description='Display function signatures.',
                               field_name='show_signature'),
@@ -406,12 +430,19 @@ class PythonInput(object):
                 simple_option(title='Show line numbers',
                               description='Show line numbers when the input consists of multiple lines.',
                               field_name='show_line_numbers'),
+                simple_option(title='Show Meta+Enter message',
+                              description='Show the [Meta+Enter] message when this key combination is required to execute commands. ' +
+                                  '(This is the case when a simple [Enter] key press will insert a newline.',
+                              field_name='show_meta_enter_message'),
                 simple_option(title='Show status bar',
                               description='Show the status bar at the bottom of the terminal.',
                               field_name='show_status_bar'),
                 simple_option(title='Show sidebar help',
                               description='When the sidebar is visible, also show this help text.',
                               field_name='show_sidebar_help'),
+                simple_option(title='Highlight parenthesis',
+                              description='Highlight matching parenthesis, when the cursor is on or right after one.',
+                              field_name='highlight_matching_parenthesis'),
             ]),
             OptionCategory('Colors', [
                 Option(title='Code',
@@ -434,17 +465,19 @@ class PythonInput(object):
         Create an `Application` instance for use in a `CommandLineInterface`.
         """
         buffers = {
-            'docstring': Buffer(),  # XXX: make docstring read only.
+            'docstring': Buffer(read_only=True),
         }
         buffers.update(self._extra_buffers or {})
 
         return Application(
             layout=create_layout(
                 self,
-                self.key_bindings_manager, self._python_prompt_control,
+                self.key_bindings_manager,
                 lexer=self._lexer,
+                input_buffer_height=self._input_buffer_height,
                 extra_buffer_processors=self._extra_buffer_processors,
-                extra_sidebars=self._extra_sidebars),
+                extra_body=self._extra_layout_body,
+                extra_toolbars=self._extra_toolbars),
             buffer=self._create_buffer(),
             buffers=buffers,
             key_bindings_registry=self.key_bindings_registry,
@@ -462,6 +495,7 @@ class PythonInput(object):
         """
         def is_buffer_multiline():
             return (self.paste_mode or
+                    self.accept_input_on_enter is None or
                     document_is_multiline_python(python_buffer.document))
 
         python_buffer = Buffer(
@@ -469,7 +503,7 @@ class PythonInput(object):
             complete_while_typing=Condition(lambda: self.complete_while_typing),
             enable_history_search=Condition(lambda: self.enable_history_search),
             tempfile_suffix='.py',
-            history=self._history,
+            history=self.history,
             completer=self._completer,
             validator=SwitchableValidator(
                 self._validator,
