@@ -11,15 +11,15 @@ from __future__ import unicode_literals
 
 from pygments.lexers import PythonTracebackLexer, PythonLexer
 from pygments.styles.default import DefaultStyle
-from pygments.token import Token
 
 from prompt_toolkit.application import AbortAction
 from prompt_toolkit.interface import AcceptAction, CommandLineInterface
 from prompt_toolkit.layout.utils import token_list_width
-from prompt_toolkit.shortcuts import create_eventloop, create_asyncio_eventloop
+from prompt_toolkit.shortcuts import create_asyncio_eventloop
 from prompt_toolkit.utils import DummyContext, Callback
 
 from .python_input import PythonInput
+from .eventloop import create_eventloop
 
 import os
 import six
@@ -94,7 +94,11 @@ class PythonRepl(PythonInput):
                            flags=self.get_compiler_flags(),
                            dont_inherit=True)
 
-        if line[0:1] == '!':
+        if line.lstrip().startswith('\x1a'):
+            # When the input starts with Ctrl-Z, quit the REPL.
+            cli.set_return_value(None)
+
+        elif line.lstrip().startswith('!'):
             # Run as shell command
             os.system(line[1:])
         else:
@@ -107,12 +111,7 @@ class PythonRepl(PythonInput):
                 locals['_'] = locals['_%i' % self.current_statement_index] = result
 
                 if result is not None:
-                    out_tokens = [
-                        (Token.Out, 'Out['),
-                        (Token.Out.Number, '%s' % self.current_statement_index),
-                        (Token.Out, ']:'),
-                        (Token, ' '),
-                    ]
+                    out_tokens = self.get_output_prompt_tokens(cli)
 
                     try:
                         result_str = '%r\n' % (result, )
@@ -145,7 +144,13 @@ class PythonRepl(PythonInput):
         # Instead of just calling ``traceback.format_exc``, we take the
         # traceback and skip the bottom calls of this framework.
         t, v, tb = sys.exc_info()
-        tblist = traceback.extract_tb(tb)[3:]
+        tblist = traceback.extract_tb(tb)
+
+        for line_nr, tb_tuple in enumerate(tblist):
+            if tb_tuple[0] == '<stdin>':
+                tblist = tblist[line_nr:]
+                break
+
         l = traceback.format_list(tblist)
         if l:
             l.insert(0, "Traceback (most recent call last):\n")
@@ -234,7 +239,7 @@ def run_config(repl, config_file='~/.ptpython/config.py'):
 
 
 def embed(globals=None, locals=None, configure=None,
-          vi_mode=False, history_filename=None,
+          vi_mode=False, history_filename=None, title=None,
           startup_paths=None, patch_stdout=False, return_asyncio_coroutine=False):
     """
     Call this to embed  Python shell at the current point in your program.
@@ -246,6 +251,7 @@ def embed(globals=None, locals=None, configure=None,
     :param vi_mode: Boolean. Use Vi instead of Emacs key bindings.
     :param configure: Callable that will be called with the `PythonRepl` as a first
                       argument, to trigger configuration.
+    :param title: Title to be displayed in the terminal titlebar. (None or string.)
     """
     assert configure is None or callable(configure)
 
@@ -277,6 +283,9 @@ def embed(globals=None, locals=None, configure=None,
                       history_filename=history_filename,
                       startup_paths=startup_paths)
 
+    if title:
+        repl.terminal_title = title
+
     if configure:
         configure(repl)
 
